/**
* Klasse, die Kommunikation mit API bündelt (nach der Initialisierung muss start() aufgerufen werden!)
*/
function LMSLink(
    boolNoAutoStop,     // Optional: Nicht automatisch beim Verlassen der Seite die Kommunikation mit dem LMS beenden
    boolNoTrack         // Optional: Nicht automatisch
) {
    Object.defineProperty(this, 'api', { value: LMSLink.findAPI(), writeable: false, enumerable: true });
    if (!boolNoAutoStop)
        this.addUnloadListener();
    if (!boolNoTrack)
        this.addPauseResumeListener();
}

/**
* Dem window-Objekt EventListener für unload und beforeunload hinzufügen, die die Kommunikation mit der API beenden
* @return undefined
*/
LMSLink.prototype.addUnloadListener = function() {
    var self = this;
    var fktUnload = function() { self.stop(); }
    var fktVisibilitychange = function() { self.sessionTime = self.sessionTime; };
    window.addEventListener('unload', fktUnload);
    window.addEventListener('beforeunload', fktUnload);
    window.addEventListener('visibilitychange', fktVisibilitychange);
}

/**
* Dem window-Objekt EventListener für focus und blur hinzufügen, die die Zeitmessung wiederaufnehmen bzw. pausieren
* @return undefined
*/
LMSLink.prototype.addPauseResumeListener = function() {
    var self = this;
    window.addEventListener('blur', function() { self.pause(); });
    window.addEventListener('focus', function() { self.resume(); });
}

// Aktueller Status: -1 => Noch nicht initialisiert, 0 => Bereit, 1 => Beendet
LMSLink.prototype.status = -1;

// Kann die Schnittstelle (noch) verwendet werden?
Object.defineProperty(LMSLink.prototype, 'ready', { get() { return !this.status; }, enumerable: true } );

/**
* API auffinden
* @return Object    LMS-API
*/
LMSLink.findAPI = function() {
    for (var w = window; w; w = (w.parent == w) ? null : w.parent) {
        if (w.API && (typeof w.API.LMSInitialize == "function"))
            return w.API;
    }
    return null;
}

/**
* LMS-Rückgabewert v in Boolean-Wert umwandeln
* @return Boolean   Umgewandelter Wert
*/
LMSLink.lms2bool = function(
    v       // Umzuwandelnder Rückgabewert von API
) {
    if (v == "true")
        return true;
    if (!v)
        return undefined;
    return false;
}

/**
* Ganzzahl v in einen String mit erzungenderweise countDigits Stellen umwandeln
* @return String    Erzeugte Darstellung von v als String
*/
LMSLink.toDigits = function(
    v,              // Zahl
    countDigits     // Gewünschte Anzahl an Ziffern
) {
    if (v >= Math.pow(10, countDigits))
        v = Math.pow(10, countDigits) - 1;
    if (!Number.isFinite(v))
        v = 0;
    var s = Math.floor(v).toString();
    while (s.length < countDigits)
        s = '0' + s;
    return s;
}

/**
* Eine Anzahl an Sekunden in eine CMITimespan-Angabe umwandeln
* @return String    Berechnete CMITimespan-Angabe
*/
LMSLink.seconds2timespan = function(
    seconds     // Anzahl an Sekunden, die in eine CMITimespan-Angabe umgewandelt werden soll
) {
    seconds = Math.round(seconds * 100) / 100;
    if (seconds >= 36000000)
        return '9999:99:99.99';
    return LMSLink.toDigits(seconds / 3600, 4) + ':' + LMSLink.toDigits((seconds / 60) % 60, 2) + ':' + LMSLink.toDigits(seconds % 60, 2) + '.' + LMSLink.toDigits((seconds * 100) % 100, 2);
}

/**
* Wandelt eine CMITimespan-Angabe in die Anzahl an Sekunden um
* @return Number        Anzahl an Sekunden
*/
LMSLink.timespan2seconds = function(
    strTimespan     // CMITimespan-Angabe der Form "HHHH:MM:SS[.mm]"
) {
    if ((typeof strTimespan != 'string') && !(strTimespan instanceof String))
        return 0;
    var a = strTimespan.split(':').map(function(z) { return (Number.isFinite(+z) ? (+z) : 0); });
    if (a.length > 1)
        a[a.length - 2] *= 60;
    if (a.length > 2)
        a[a.length - 3] *= 3600;
    var v = a.reduce(function(a, b) { return a + b; });
    return (v > 36000060) ? Infinity : v;
}

/**
* Initialisierung durchführen, sofern dies noch aussteht
* @return Boolean       Wurde (jetzt oder zuvor) eine erfolgreiche Initialisierung durchgeführt
*/
LMSLink.prototype.start = function(
    boolAllwaysResume       // Immer auch this.resume aufrufen, auch dann, wenn das Fenster nicht aktiv ist
) {
    if (this.status >= 0)
        return true;
    if (!this.api)
        return false;
    if (LMSLink.lms2bool(this.api.LMSInitialize(""))) {
        this.status = 0;
        var time = LMSLink.timespan2seconds(this.get('cmi.core.session_time'));
        if (time > 1)
            this._time += time;
        switch (this.entry) {
            case 'ab-initio':
                if (typeof this.onnew == 'function')
                    this.onnew.call(this);
                else {
                    this.lessonStatus = 'incomplete';
                    this.scoreMin = 0;
                    this.scoreMax = 100;
                }
                break;
            case 'resume':
                if (typeof this.onresume == 'function')
                    this.onresume.call(this);
                break;
        }

        if (boolAllwaysResume || document.hasFocus())
            this.resume();
        return true;
    }
    return false;
}

/**
* Kommunikation mit LMS beenden, sofern dies noch aussteht
* @return Boolean       Wurde (jetzt oder zuvor) die Kommunikation förmlich beendet?
*/
LMSLink.prototype.stop = function() {
    if (this.status < 0)
        return false;
    if (this.status > 0)
        return true;
    if (!this.api)
        return false;
    this.pause();
    if (LMSLink.lms2bool(this.api.LMSFinish(""))) {
        this.status = 1;
        return true;
    }
    return false;
}

// Anzahl an Sekunden, die diese SCO bereits genutzt wurde
LMSLink.prototype._time = 0;

// Zeitstempel des Zeitpunktes, an dem die letzte Bearbeitungspause beendet wurde
LMSLink.prototype._lastTimestamp = null;

// Funktion, die beim Starten oder Wiederaufnehmen der Bearbeitung des SCOs ausgeführt werden soll
LMSLink.prototype.onrestart = undefined;

// Funktion, die beim Beenden oder Pausieren der Bearbeitung des SCOs ausgeführt werden soll
LMSLink.prototype.onpause = undefined;

// Funktion, die aufgerufen werden soll, wenn das Lernpaket zum ersten Mal oder neu bearbeitet wird
LMSLink.prototype.onnew = undefined;

// Funktion, die beim Wiederladen des SCOs ausgeführt werden soll
LMSLink.prototype.onresume = undefined;

/**
* Bearbeitung des aktuellen SCOs pausieren
* @return undefined
*/
LMSLink.prototype.pause = function() {
    if (this.status)
        return;
    if (typeof this.onpause == 'function')
        this.onpause.call(this);
    if (this._lastTimestamp instanceof Date) {
        this._time += ((new Date()).getTime() - this._lastTimestamp.getTime()) / 1000;
        this._lastTimestamp = null;
    }
    this.sessionTime = this.sessionTime;
}

/**
* Bearbeitung des aktuellen SCOs (wieder-)aufnehmen
* @return undefined
*/
LMSLink.prototype.resume = function() {
    if (this.status)
        return;
    this._lastTimestamp = new Date();
    if (typeof this.onrestart == 'function')
        this.onrestart.call(this);
}

// Ist das Lernmodul gerade aktiv?
Object.defineProperty(LMSLink.prototype, 'active', { enumerable: true,
    get() { return (this._lastTimestamp !== null); },
    set(v) { if (v) this.resume(); else this.pause(); }
});

// Anzahl an Sekunden, die diesmal schon am SCO gearbeitet wurde
Object.defineProperty(LMSLink.prototype, 'sessionTime', { enumerable: true,
    get() {
        var seconds = this._time;
        if (this._lastTimestamp instanceof Date)
            seconds += ((new Date()).getTime() - this._lastTimestamp.getTime()) / 1000;
        return seconds;
    },
    set(v) {
        this.set('cmi.core.session_time', LMSLink.seconds2timespan(v));
    }
});

// Anzahl an Sekunden, die zuvor schon insgesamt mit diesem Lernpaket verbracht wurde
Object.defineProperty(LMSLink.prototype, 'totalTime', { enumerable: true,
    get() {
        return LMSLink.timespan2seconds(this.get('cmi.core.total_time'));
    }
});

// Maximal erlaubte Bearbeitungszeit in Sekunden
Object.defineProperty(LMSLink.prototype, 'maxTime', { enumerable: true,
    get() {
        return LMSLink.timespan2seconds(this.get('cmi.student_data.max_time_allowed'));
    }
});

// Was soll passieren, wenn die maximale Bearbeitungszeit abgelaufen ist?
Object.defineProperty(LMSLink.prototype, 'timeoutAction', { enumerable: true,
    get() {
        return this.get('cmi.student_data.time_limit_action');
    }
});

// Wird automatisch beendet, wenn die maximale Bearbeitungszeit abgelaufen ist?
Object.defineProperty(LMSLink.prototype, 'timeoutExit', { enumerable: true,
    get() {
        var s = this.timeoutAction;
        if (s === undefined)
            return undefined;
        return s.length ? (s.indexOf('exit') > -1) : false;
    }
});

// Soll eine Meldung angezeigt werden, wenn die maximale Bearbeitungszeit abgelaufen ist?
Object.defineProperty(LMSLink.prototype, 'timeoutMessage', { enumerable: true,
    get() {
        var s = this.timeoutAction;
        if (s === undefined)
            return undefined;
        return s.length ? !(s.indexOf('no message') > -1) : false;
    }
});

// Nummer des letzten Fehlers (als Number)
Object.defineProperty(LMSLink.prototype, 'lastError', { enumerable: true,
    get() {
        if (!this.api)
            return -1;
        return +this.api.LMSGetLastError();
    }
});

// Beschreibung des letzten Fehlers
Object.defineProperty(LMSLink.prototype, 'lastErrorDesc', { enumerable: true,
    get() {
        if (!this.api)
            return 'Die Schnittstelle zur Kommunikation mit dem übergeordneten System konnte nicht gefunden werden.';
        return this.api.LMSGetErrorString(this.api.LMSGetLastError());
    }
});

// Weitergehende Erklärum zum letzten Fehlers
Object.defineProperty(LMSLink.prototype, 'lastErrorHelp', { enumerable: true,
    get() {
        if (!this.api)
            return '';
        return this.api.LMSGetDiagnostic(this.api.LMSGetLastError());
    }
});

/**
* Den Wert von key auslesen
* @return String    Gelesener Wert
*/
LMSLink.prototype.get = function(
    key         // Name des auszulesenden Wertes
) {
    if (this.status)
        return undefined;
    var result = this.api.LMSGetValue(key);
    if ((result == "") && this.lastError)
        return undefined;
    return result;
}

/**
* Wert value unter dem Namen key speichern
* @return Boolean       Hat das Speichern geklappt?
*/
LMSLink.prototype.set = function(
    key,        // Name des abzuspeichernden Wertes
    value       // Anzuspeichernde Wert
) {
    if (this.status)
        return false;
    return (LMSLink.lms2bool(this.api.LMSSetValue(key, value)) && LMSLink.lms2bool(this.api.LMSCommit("")));
}

// ID des Benutzers
Object.defineProperty(LMSLink.prototype, 'studentID', { enumerable: true, get() { return this.get('cmi.core.student_id'); } });

// Name des Benutzers
Object.defineProperty(LMSLink.prototype, 'studentName', { enumerable: true, get() { return this.get('cmi.core.student_name'); } });

// Vorname des Benutzers
Object.defineProperty(LMSLink.prototype, 'studentFirstName', { enumerable: true,
    get() {
        var s = this.studentName;
        if (s === undefined)
            return undefined;
        var index = s.indexOf(',');
        if (index > -1)
            return s.substr(index + 1).trim();
        s = s.trim();
        index = s.indexOf(' ');
        if (index > -1)
            return s.substr(0, index);
        return s;
    }
});

// Aktueller „Ort“ im SCO
Object.defineProperty(LMSLink.prototype, 'lessonLocation', { enumerable: true,
    get() { return this.get('cmi.core.lesson_location'); },
    set(v) { this.set('cmi.core.lesson_location', v); }
});

// Findet eine Bewertung statt?
Object.defineProperty(LMSLink.prototype, 'assessing', { enumerable: true, get() { v = this.get('cmi.core.credit'); return (v === undefined) ? v : (v != 'no-credit'); } });

// Aktueller Status des SCO: "passed", "completed", "failed", "incomplete", "browsed", "not attempted"
Object.defineProperty(LMSLink.prototype, 'lessonStatus', { enumerable: true,
    get() { return this.get('cmi.core.lesson_status'); },
    set(v) { this.set('cmi.core.lesson_status', v); }
});

// Wurde SCO schon einmal aufgerufen?
Object.defineProperty(LMSLink.prototype, 'entry', { enumerable: true, get() { return this.get('cmi.core.entry'); } });

// Erreichte Punkteanzahl
Object.defineProperty(LMSLink.prototype, 'score', { enumerable: true,
    get() { var v = this.get('cmi.core.score.raw'); return (v === undefined) ? v : (+v); },
    set(v) { this.set('cmi.core.score.raw', v); }
});

// Minimal erreichbare Punkteanzahl
Object.defineProperty(LMSLink.prototype, 'scoreMin', { enumerable: true,
    get() { var v = this.get('cmi.core.score.min'); return (v === undefined) ? v : (+v); },
    set(v) { this.set('cmi.core.score.min', v); }
});

// Maximal erreichbare Punkteanzahl
Object.defineProperty(LMSLink.prototype, 'scoreMax', { enumerable: true,
    get() { var v = this.get('cmi.core.score.max'); return (v === undefined) ? v : (+v); },
    set(v) { this.set('cmi.core.score.max', v); }
});

// Zum Bestehen nötige Punkteanzahl
Object.defineProperty(LMSLink.prototype, 'scoreMastery', { enumerable: true,
    get() { var v = this.get('cmi.student_data.mastery_score'); return (v === undefined) ? v : (+v); },
});

// Art der Bearbeitung: "browse", "normal" oder "review"
Object.defineProperty(LMSLink.prototype, 'lessonMode', { enumerable: true, get() { return this.get('cmi.core.lesson_mode'); } });

// Wie das Lernpaket verlassen wird
Object.defineProperty(LMSLink.prototype, 'exit', { enumerable: true, set(v) {
    this.set('cmi.core.exit', v);
} });

// Daten (String), welche bei der Weiterbearbeitung des Lernpaketes berücksichtigt werden sollen
Object.defineProperty(LMSLink.prototype, 'suspendData', { enumerable: true,
    get() { return this.get('cmi.suspend_data'); },
    set(v) { this.set('cmi.suspend_data', v); }
});

// Zusätzlich vom LMS beim Start übergebene Daten (String)
Object.defineProperty(LMSLink.prototype, 'launchData', { enumerable: true,
    get() { return this.get('cmi.launch_data'); }
});

// Benutzerkommentar
Object.defineProperty(LMSLink.prototype, 'comments', { enumerable: true,
    get() { return this.get('cmi.comments'); },
    set(v) { this.set('cmi.comments', v); }
});

// Kommentar vom LMS
Object.defineProperty(LMSLink.prototype, 'commentsFromLms', { enumerable: true,
    get() { return this.get('cmi.comments_from_lms'); }
});

// Gewünschte Lautstärke (Integer)
Object.defineProperty(LMSLink.prototype, 'preferenceAudio', { enumerable: true,
    get() { var v = this.get('cmi.student_preference.audio'); return (v === undefined) ? v : (+v); },
    set(v) { this.set('cmi.student_preference.audio', v); }
});

// Gewünschte Sprache
Object.defineProperty(LMSLink.prototype, 'preferenceLanguage', { enumerable: true,
    get() { return this.get('cmi.student_preference.language'); },
    set(v) { this.set('cmi.student_preference.language', v); }
});

// Gewünschte relative Geschwindigkeit (Integer)
Object.defineProperty(LMSLink.prototype, 'preferenceSpeed', { enumerable: true,
    get() { var v = this.get('cmi.student_preference.speed'); return (v === undefined) ? v : (+v); },
    set(v) { this.set('cmi.student_preference.speed', v); }
});

// Ob und wie Untertitel angezeigt werden sollen (Integer)
Object.defineProperty(LMSLink.prototype, 'preferenceText', { enumerable: true,
    get() { var v = this.get('cmi.student_preference.text'); return (v === undefined) ? v : (+v); },
    set(v) { this.set('cmi.student_preference.text', v); }
});
